xquery version "1.0";

(: Импорт стандартных модулей :)
declare namespace request="http://exist-db.org/xquery/request";
declare namespace response="http://exist-db.org/xquery/response";
declare namespace session="http://exist-db.org/xquery/session";

(: Импорт модулей приложения :)
import module namespace xrx_example = "http://iu5.bmstu.ru/edu/xrx_example" at "module.xqm";

(: Формат в котором результат должен отображаться в браузере :)
declare option exist:serialize "method=xhtml media-type=text/html indent=no";

(: Идентификатор модуля :)
declare variable $module_id {"computer"};
(: Версия модуля :)
declare variable $module_ver {"1.1"};

(: Название сессионной переменной для временного хранения данных формы :)
declare variable $current_data {"current_data"};

(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
(: Создание нового элемента данных :) 
(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
declare function local:NewData($idParam as xs:string) as node()
{
let $FirstUpgradeId := xrx_example:MakeUniqueName()
return 
<computer id="{$idParam}">
  <computer_name/>
  <processor_id/>
  <op/>
  <hdd/>
  <server>false</server>
  <upgrades>
    <upgrade/>
  </upgrades>
</computer>  
};

(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
(: Создание нового элемента данных для модернизации :) 
(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
declare function local:NewUpgradeData() as node()
{
<upgrades>
  <upgrade id="">
    <upgrade_date/>
    <upgrade_description/>
  </upgrade>
</upgrades>
};

(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
(: Сообщения об ошибках :) 
(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
declare function local:CheckErrors($Data as node()) as node()*
{
(
  if(($Data//op mod 2) != 0)
    then (<p>Объем ОП должен быть четным числом</p>)
    else ()
),(
  if(($Data//hdd mod 2) != 0)
    then (<p>Объем жесткого диска должен быть четным числом</p>)
    else ()
)
};

(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
(: Сохранение данных :) 
(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
declare function local:SaveData($DataListURI as xs:string) as node()*
{
let 
  (: В сессионной переменной хранится строка, поэтому необходимо сделать преобразование к элементу :)
  $TempData := element temp {session:get-attribute($current_data)},
  $Data := $TempData//computer, 
  $id := xs:string($Data/@id),
  $Errors := local:CheckErrors($Data)
return 
  if(count($Errors) = 0)
  (: Сохранение данных :)
  then
  (
    xrx_example:SaveFileInCollection(xrx_example:PathURI_data_list_in_db($module_id), $id, $Data),  
    xrx_example:redirect($DataListURI) 
  )
  (: Сообщения об ошибках :)
  else
  (
    xrx_example:ShowErrorReturnForm($module_id, $module_ver, "Ошибка при сохранении данных", $Errors)
  )
};

(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
(: Предварительное сохранение данных :) 
(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
declare function local:PreSaveData() as node()*
{
(: Если сессия не создана, то она создается :)
session:create(),
session:set-attribute($current_data, ""),
let 
  $PostData := request:get-data(),
  $Data := fn:root($PostData)//computer 
return
  session:set-attribute($current_data, $Data)
};

(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
(: Удаление данных :) 
(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
declare function local:DeleteData($deleteidParam as xs:string, $DataListURI as xs:string) as node()*
{
  xrx_example:DeleteFileInCollection(xrx_example:PathURI_data_list_in_db($module_id), $deleteidParam),
  xrx_example:redirect($DataListURI)
};

(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
(: Основной запрос :) 
(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
let 
  $newParam := xs:string(request:get-parameter("new","0")),
  $newUpgradeParam := xs:string(request:get-parameter("newupgrade","0")),
  $saveParam := xs:string(request:get-parameter("save","0")),
  $presaveParam := xs:string(request:get-parameter("presave","0")),
  $idParam := xs:string(request:get-parameter("id","0")),
  $nameParam := xs:string(request:get-parameter("processor_name","0")),
  $deleteidParam := xs:string(request:get-parameter("deleteid","0")),
  $DataListURI := xrx_example:PathURI_list($module_id, $module_ver)
return
  if($newParam = '1')
    (: Создание нового экземпляра :)
    then 
    (
      let $newDataId := xrx_example:MakeUniqueName()
      return local:NewData($newDataId)
    )
  else if($newUpgradeParam = '1')
    (: Создание нового экземпляра для upgrade :)
    then 
    (
	  let $NewUpgrade := local:NewUpgradeData()
	  return $NewUpgrade 
    )
  else if($saveParam = '1')
    (: Сохранение данных :)
    then 
    (
      local:SaveData($DataListURI)
    )
  else if($presaveParam = '1')
    (: Сохранение данных во временной переменной :)
    then 
    (
      local:PreSaveData()
    )
  else if($deleteidParam != '0')
    (: Удаление данных :)
    then 
    (
      local:DeleteData($deleteidParam, $DataListURI)	
    )
  else if($idParam != '0')
    (: Чтение данных для редактирования в форме :)
    then
    (
	  let 
        $DataPath := concat(xrx_example:PathURI_data_list_in_db($module_id), xrx_example:MakeUniqueFileNameExtXml($idParam)),
        $FileData := doc($DataPath)//computer
      return $FileData
    )
  else ()	
(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
