xquery version "1.0";

(: Импорт стандартных модулей :)
declare namespace request="http://exist-db.org/xquery/request";
declare namespace response="http://exist-db.org/xquery/response";
declare namespace session="http://exist-db.org/xquery/session";

(: Импорт модулей приложения :)
import module namespace xrx_example = "http://iu5.bmstu.ru/edu/xrx_example" at "module.xqm";

(: Формат в котором результат должен отображаться в браузере :)
declare option exist:serialize "method=xhtml media-type=text/html indent=no";

(: Идентификатор модуля :)
declare variable $module_id {"processor"};
(: Версия модуля :)
declare variable $module_ver {"1.2"};

(: Для вызова модуля компьютера :)
declare variable $module_computer_id {"computer"};
declare variable $module_computer_ver {"1.1"};

(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
(: Создание элемента данных для процессора :) 
(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
declare function local:NewData($idParam as xs:string, $nameParam as xs:string) as node()
{
<processor id="{$idParam}">
  <processor_name>{$nameParam}</processor_name>	
</processor>  
};

(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
(: Сообщения об ошибках :) 
(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
declare function local:CheckErrors($nameParam as xs:string) as node()*
{
if(string-length(normalize-space($nameParam)) = 0)
  then (<p>Наименование не может быть пустой строкой</p>)
  else ()
};

(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
(: Сохранение данных :) 
(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
declare function local:SaveData($idParam as xs:string, $nameParam as xs:string, $DataListURI as xs:string) as node()*
{
let
  $Errors := local:CheckErrors($nameParam)
return 
  if(count($Errors) != 0)
  (: Сообщения об ошибках :)
  then
  (
    xrx_example:ShowErrorReturnForm($module_id, $module_ver, "Ошибка при сохранении данных", $Errors)
  )

  (: Сохранение данных :)
  else
  (
    (
    if($idParam = 'new')
    then 
    (: Добавление данных :)
    (
      let 
        $newDataId := xrx_example:MakeUniqueName(),
        $newData:= local:NewData($newDataId, $nameParam)
      return
        update insert $newData into doc(xrx_example:ProcessorsFileName())//processors
    )
    else 
    (: Обновление данных :)
    (
      let 
        $newData:= local:NewData($idParam, $nameParam)
      return
        update replace doc(xrx_example:ProcessorsFileName())//processor[@id=$idParam] with $newData
    )
	),(
    (: После успешного добавления или обновления данных переход к списку :) 
    xrx_example:redirect($DataListURI)
	)
  )
};

(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
(: Сообщения об ошибках :) 
(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
declare function local:CheckDeleteErrors($deleteidParam as xs:string) as node()*
{
(: Проверка того, что удаляемый тип процессора не используется в компьютерах :)
let 
  $ComputersList := collection(xrx_example:PathURI_data_list_in_db($module_computer_id)),
  $cnt := count($ComputersList//computer[processor_id = $deleteidParam])
return
  if($cnt > 0)
    then (<p>Этот тип процессора не может быть удален, так как он используется при описании {$cnt} компьютера(ов)</p>)
    else ()
};

(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
(: Удаление данных :) 
(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
declare function local:DeleteData($deleteidParam as xs:string, $DataListURI as xs:string) as node()*
{
let
  $Errors := local:CheckDeleteErrors($deleteidParam)
return 
  if(count($Errors) != 0)
  (: Сообщения об ошибках :)
  then
  (
    xrx_example:ShowErrorReturnList($module_id, $module_ver, "Ошибка при удалении данных", $Errors)
  )
  (: Удаление данных :)
  else
  (
    (
    let $del := doc(xrx_example:ProcessorsFileName())//processor[@id=$deleteidParam] 
    return update delete $del
	),(
    (: После удаления данных переход к списку :) 
    xrx_example:redirect($DataListURI)
	)
  )
};

(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
(: Основной запрос :) 
(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
let 
  $newParam := xs:string(request:get-parameter("new","0")),
  $saveParam := xs:string(request:get-parameter("save","0")),
  $idParam := xs:string(request:get-parameter("id","0")),
  $nameParam := xs:string(request:get-parameter("processor_name","0")),
  $deleteidParam := xs:string(request:get-parameter("deleteid","0")),
  $listParam := xs:string(request:get-parameter("list","0")), 
  $DataListURI := xrx_example:PathURI_list($module_id, $module_ver)
return
  if($saveParam = '1')
    (: Сохранение данных :)
    then 
    (
      local:SaveData($idParam, $nameParam, $DataListURI)
    )
  else if($deleteidParam != '0')
    (: Удаление данных :)
    then 
    (
      local:DeleteData($deleteidParam, $DataListURI)	
    )
  else if($listParam = '1')
    (: Выборка данных из справочника с сортировкой по наименованию :)
    then 
    (
	  for $data in doc(xrx_example:ProcessorsFileName())//processor
	  order by $data/processor_name
	  return $data
    )
  else ()	
(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
