xquery version "1.0";

(: Импорт стандартных модулей :)
declare namespace request="http://exist-db.org/xquery/request";
declare namespace session="http://exist-db.org/xquery/session";

(: Импорт модулей приложения :)
import module namespace xrx_example = "http://iu5.bmstu.ru/edu/xrx_example" at "module.xqm";

(: Формат в котором результат должен отображаться в браузере :)
declare option exist:serialize "method=xhtml media-type=text/html indent=no";

(: Идентификатор модуля :)
declare variable $module_id {"computer"};
(: Версия модуля :)
declare variable $module_ver {"1.1"};

(: Для вызова модуля процессора :)
declare variable $module_processor_id {"processor"};
declare variable $module_processor_ver {"1.2"};

(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
(: Модели данных формы :) 
(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
declare function local:CreateXFormsModels($DataReadURI as xs:string, $DataSaveURI as xs:string, $ProcessorReadURI as xs:string, $DataNewUpgradeURI as xs:string, $DataPreSaveURI as xs:string) as node()*
{
(
(: Основная модель данных :)
<xforms:model xmlns:xforms="http://www.w3.org/2002/xforms" id="computer_model">

  <!-- Основные данные (о компьютере) -->
  <xforms:instance id="computer_instance" xmlns="" src="{$DataReadURI}"/>

  <!-- Данные для добавления нового обновления -->
  <xforms:instance id="upgrade_instance" xmlns="" src="{$DataNewUpgradeURI}"/>
 
  <!-- Наименование компьютера - обязательное поле и введенные данные не состоят только из пробелов -->
  <xforms:bind nodeset="//computer_name" required="true()" constraint="string-length(normalize-space(.)) &gt; 0" />

  <!-- Тип процессора - обязательное поле -->
  <xforms:bind nodeset="//processor_id" required="true()" />  
  
  <!-- ОП - обязательное поле целого типа в диапазоне от 0 до 4096 -->
  <xforms:bind nodeset="//op" type="xforms:int" required="true()" constraint="(. &gt;= 1) and (. &lt;= 4096)" />

  <!-- Объем жесткого диска - обязательное поле целого типа в диапазоне от 0 до 1000 -->
  <xforms:bind nodeset="//hdd" type="xforms:int" required="true()" constraint="(. &gt;= 1) and (. &lt;= 1000)" />
  
  <!-- Признак сервера - логического типа -->
  <xforms:bind nodeset="//server" type="xforms:boolean" />

  <!-- Дата модернизации - обязательное поле типа дата -->
  <xforms:bind nodeset="//upgrade_date" type="xforms:date" required="true()" />
  
  <!-- Описание модернизации - обязательное поле и введенные данные не состоят только из пробелов -->
  <xforms:bind nodeset="//upgrade_description" required="true()" constraint="string-length(normalize-space(.)) &gt; 0" />
  
  <!-- Отправка формы -->
  <xforms:submission replace="none" action="{$DataPreSaveURI}" method="post" id="submit_id" includenamespaceprefixes=""/>
</xforms:model>		
),(
(: Данные для выбора типов процессоров :)
<xforms:model xmlns:xforms="http://www.w3.org/2002/xforms" id="processor_model">
  <xforms:instance id="processor_instance" xmlns="" src="{$ProcessorReadURI}"/>
</xforms:model>		
)
};

(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
(: Редактирование формы :) 
(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
declare function local:EditForm($ListURI as xs:string, $DataPreSaveURI as xs:string, $DataSaveURI as xs:string) as node()
{
<div xmlns:xforms="http://www.w3.org/2002/xforms" xmlns:ev="http://www.w3.org/2001/xml-events">
  <table border="1" align="center" >

    <tr>	
      <th colspan="2" align="right">Наименование компьютера:</th>	
      <td>
        <xforms:input model="computer_model" ref="instance('computer_instance')//computer_name">
          <xforms:alert>Поле не может быть пустым</xforms:alert>
        </xforms:input>
      </td>
    </tr>

    <tr>	
      <th colspan="2" align="right">Тип процессора:</th>	
      <td>
        <xforms:select1 model="computer_model" ref="processor_id">
          <xforms:alert>Необходимо выбрать тип процессора</xforms:alert>
          <xforms:itemset model="processor_model" nodeset="//processor">
            <xforms:value ref="@id"/>
            <xforms:label ref="processor_name"/>
          </xforms:itemset>
        </xforms:select1>
      </td>
    </tr>
	
    <tr>	
      <th rowspan="2" align="right">Объем ОП:</th>	
      <th align="right">(в Мб):</th>	
      <td>
        <xforms:input model="computer_model" ref="op">
          <xforms:alert>Значение должно быть целым числом в диапазоне от 1 до 4096</xforms:alert>
        </xforms:input>
      </td>
    </tr>

    <tr>	
      <th align="right">(в Гб):</th>	
      <td>
        <xforms:output model="computer_model" value="concat(string(round(op div 1024)), ' [без округления: ', string(op div 1024), ']')">
          <xforms:alert>Значение вычисляется на основе поля "Объем ОП (в Мб)"</xforms:alert>
        </xforms:output>
      </td>
    </tr>
	
    <tr>	
      <th colspan="2" align="right">Объем жесткого диска (в Гб):</th>	
      <td>
        <xforms:input model="computer_model" ref="hdd">
          <xforms:alert>Значение должно быть целым числом в диапазоне от 1 до 1000</xforms:alert>
        </xforms:input>
      </td>
    </tr>

    <tr>	
      <th colspan="2" align="right">Используется как сервер:</th>	
      <td>
        <xforms:input model="computer_model" ref="server">
          <xforms:hint>Если компьютер используется в качестве сервера, то флаг должен быть включен</xforms:hint>
        </xforms:input>
      </td>
    </tr>

    <tr>	
      <td colspan="3">
        <h2>Список модернизаций</h2>

        <ol>
          <xforms:repeat id="repeat_upgrades" model="computer_model" nodeset="//upgrades/upgrade[@id]" appearance="compact">
            <li>

              <xforms:input model="computer_model" ref="upgrade_date">
                <xforms:label>Дата:</xforms:label>
                <xforms:alert>Поле не может быть пустым и должно содержать дату</xforms:alert>
              </xforms:input>
			
              <xforms:textarea model="computer_model" ref="upgrade_description">
                <xforms:label>Описание:</xforms:label>
                <xforms:alert>Поле не может быть пустым</xforms:alert>
              </xforms:textarea>
							
              <!-- Удаление модернизации -->
              <xforms:trigger>
                <xforms:label>Удалить</xforms:label>
                <xforms:delete ev:event="DOMActivate" model="computer_model" nodeset="." />				
              </xforms:trigger>

            </li>
          </xforms:repeat>
        </ol>

        <!-- Добавление модернизации -->
        <xforms:trigger>
          <xforms:label>Добавить</xforms:label>
          <!-- Действия -->
          <xforms:action ev:event="DOMActivate">
            <!-- Добавление элемента из отдельного instance -->
            <xforms:insert model="computer_model" nodeset="//upgrades/upgrade" at="last()" position="after" origin="instance('upgrade_instance')//upgrade" />
            <!-- Генерация id - id генерируется как номер элемента в списке и текущие дата и время -->
            <xforms:setvalue model="computer_model" ref="//upgrade[last()]/@id" value="concat(string(index('repeat_upgrades')), '_', string(now()))"/>
		  </xforms:action>
        </xforms:trigger>
		
      </td>
    </tr>
		
    <tr>	
      <td colspan="3" align="center">

        <xforms:trigger>
          <xforms:label>Сохранить</xforms:label>
          <xforms:action ev:event="DOMActivate" if="is-valid(instance('computer_instance'))">
			<!-- Данные передаются на сервер и сохраняются в сессионной переменной -->
			<xforms:send submission="submit_id" />
			<!-- Данные читаются из сессионной переменной, проверяются и сохраняются в БД -->		
            <xforms:load show="replace" resource="{$DataSaveURI}"/>
          </xforms:action>
        </xforms:trigger>
	
        <xforms:trigger>
          <xforms:label>Отменить</xforms:label>
          <xforms:load ev:event="DOMActivate" show="replace" resource="{$ListURI}"/>
        </xforms:trigger>
	
      </td>
    </tr>
  </table>
</div>
};

(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
(: Основной запрос :) 
(: ++++++++++++++++++++++++++++++++++++++++++++++ :)

(: Если не заполнен справочник типов процессоров, то редактирование компьютеров невозможно (нет данных для выпадающего списка процессоров) :)
if (xrx_example:ModuleDataCount($module_processor_id, $module_processor_ver) = 0)
then
(: Вывод сообщения об ошибке :)
(
  xrx_example:ShowErrorButton("Невозможно редактировать данные о компьютере пока не заполнен справочник типов процессоров", "Вернуться в главное меню", $xrx_example:MainScript)
)
else
(: Выполнение основных действий :)
(
let
$newParam := xs:string(request:get-parameter("new","0")),
$idParam := request:get-parameter("id","0"),
$DataReadURI := xrx_example:PathURI_data_edit($module_id, $module_ver, $idParam, $newParam),
$DataPreSaveURI := xrx_example:PathURI_data_presave($module_id, $module_ver),
$DataSaveURI := xrx_example:PathURI_data_save($module_id, $module_ver),
$ListURI := xrx_example:PathURI_list($module_id, $module_ver),
$ProcessorReadURI := xrx_example:PathURI_data_list($module_processor_id, $module_processor_ver),
$DataNewUpgradeURI := concat(xrx_example:PathURI_data($module_id, $module_ver), "?newupgrade=1"),
$CurrentCssURI := xrx_example:PathURI_current_css(),
$XFormsCssURI := xrx_example:PathURI_xforms_css(),
$form :=
<html xmlns="http://www.w3.org/1999/xhtml" xmlns:xforms="http://www.w3.org/2002/xforms" xmlns:ev="http://www.w3.org/2001/xml-events">
  <head>
    <meta http-equiv="Content-Type" content="text/html;charset=UTF-8"/>
    <style type="text/css">
      <!-- Добавление стилей CSS с помощью XInclude -->
      <xi:include xmlns:xi="http://www.w3.org/2001/XInclude" href="{$CurrentCssURI}"/>
      <xi:include xmlns:xi="http://www.w3.org/2001/XInclude" href="{$XFormsCssURI}"/>
    </style>
    <!-- Модель создается в секции head -->
    { local:CreateXFormsModels($DataReadURI, $DataSaveURI, $ProcessorReadURI, $DataNewUpgradeURI, $DataPreSaveURI) }
  </head>
  <body>
    <h1>Справочник компьютеров</h1>
    <!-- Поля редактирования формы -->
    { local:EditForm($ListURI, $DataPreSaveURI, $DataSaveURI) }
  </body>
</html>
(: сгенерированный XForms-документ обрабатывается с помощью XSLT-преобразования для получения HTML-документа :)
return xrx_example:transform_xsltforms($form)
)
