xquery version "1.0";

(: Импорт стандартных модулей :)
declare namespace request="http://exist-db.org/xquery/request";
declare namespace session="http://exist-db.org/xquery/session";

(: Импорт модулей приложения :)
import module namespace xrx_example = "http://iu5.bmstu.ru/edu/xrx_example" at "module.xqm";

(: Формат в котором результат должен отображаться в браузере :)
declare option exist:serialize "method=xhtml media-type=text/html indent=no";

(: Идентификатор модуля :)
declare variable $module_id {"report"};
(: Версия модуля :)
declare variable $module_ver {""};

(: Для вызова модуля процессора :)
declare variable $module_processor_id {"processor"};
declare variable $module_processor_ver {"1.2"};

(: Для вызова модуля компьютера :)
declare variable $module_computer_id {"computer"};
declare variable $module_computer_ver {"1.1"};

(: Название сессионной переменной для хранения данных с параметрами отчета :)
declare variable $report_params_data {"report_params_data"};

(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
(: Сообщения об ошибках при вводе параметров отчета :) 
(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
declare function local:CheckReportErrors($Data as node()) as node()*
{
(
(: Проверка диапазона имеет смысл, только если заданы оба значения :)
if(($Data//op1 castable as xs:integer) and ($Data//op2 castable as xs:integer))
  then
  (
    let 
      $op1 := xs:integer($Data//op1),
      $op2 := xs:integer($Data//op2)
    return  
      if($op1 > $op2)
        then (<p>Минимальное ограничение для объема ОП должно быть меньше или равно максимальному</p>)
        else ()
  )
  else ()
),(
(: Проверка диапазона имеет смысл, только если заданы оба значения :)
if(($Data//upgrade_date1 castable as xs:date) and ($Data//upgrade_date2 castable as xs:date))
  then
  (
    let 
      $upgrade_date1 := xs:date($Data//upgrade_date1),
      $upgrade_date2 := xs:date($Data//upgrade_date2)
    return  
      if($upgrade_date1 > $upgrade_date2)
        then (<p>Минимальное ограничение для даты модернизации должно быть меньше или равно максимальному</p>)
        else ()
  )
  else ()
)
};

(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
(: Проверка ошибок и формирование отчета :) 
(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
declare function local:CheckErrorsAndMakeReport($DataListURI as xs:string) as node()*
{
let 
  (: В сессионной переменной хранится строка, поэтому необходимо сделать преобразование к элементу :)
  $TempData := element temp {session:get-attribute($report_params_data)},
  $Data := $TempData//report_params, 
  $Errors := local:CheckReportErrors($Data)
return 
  if(count($Errors) = 0)
  (: Формирование отчета :)
  then
  (
    local:Report($Data) 
  )
  (: Сообщения об ошибках :)
  else
  (
    xrx_example:ShowErrorReturnForm("", "", "Ошибка при вводе параметров отчета", $Errors)
  )
};

(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
(: Формирование запроса для отчета. 
Запрос формируется в виде строки, содержащей XQuery-запрос.
В дальнейшем этот запрос будет динамически интерпертирован и выполнен с помощью функции util:eval 
(аналогично динамическому SQL) :) 
(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
declare function local:MakeReportQuery($Data as node()) as xs:string
{
let 
  $t1 := concat("(: report_data - корневой элемент данных :)", "&#13;", 
                "<report_data>", "&#13;", "{ ", "&#13;",
                "  let ", "&#13;",
				   "  (: Документ, содержащий информацию о процессорах :)", "&#13;",
				   "  $Processors := doc('", xrx_example:ProcessorsFileName(), "')//processors, ", "&#13;",
   				   "  (: Коллекция, содержащая даные о компьютерах :)", "&#13;",
				   "  $ComputersList := collection('", xrx_example:PathURI_data_list_in_db("computer"), "') ", "&#13;",
				   "  for ", "&#13;",
				   "    (: Перебор всех всех файлов в коллекции, из каждого файла читается корневой элемент computer :)", "&#13;",
				   "    $Computer in $ComputersList//computer ", "&#13;",
                   "  let ", "&#13;", 
				   "    (: Название компьютера выделяется в отдельную переменную для удобства сортировки :)", "&#13;",
				   "    $computer_name := $Computer/computer_name, ", "&#13;",
				   "    (: Для каждого компьютера по id процессора читается название процессора :)", "&#13;",
				   "    $processor_name := $Processors//processor[@id=$Computer/processor_id]/processor_name ", "&#13;",
				   "  (: where - условия выборки, (1=1) - заглушка для where, если далее не задано никаких условий :)", "&#13;",
				   "  where (1=1) ", "&#13;"),
   
  $t2 := concat($t1,  
         if($Data/computer_name = "") then ("") 
         else 
		 (
           concat(
		   "    (: Если пользователь указал часть названия компьютера, то этот параметр должен входить как подстрока в названия компьютеров :)", "&#13;",
		   "    and (fn:contains($Computer/computer_name, '", $Data/computer_name, "')) ", "&#13;" 
		   )
         )),		 
  
  $t3 := concat($t2,  
         if($Data/processor_id = "") then ("") 
         else 
		 (
           concat(
		   "    (: Параметр отчета 'процессор' возвращается в виде набора id-элементов, соединенных пробелами :)", "&#13;",  
		   "    (: Поэтому нужно проверить, не входит ли id текущего компьютера в эту строку :)", "&#13;",  
		   "    and (fn:contains('", $Data/processor_id, "', $Computer/processor_id)) ", "&#13;"
		   )
         )),		 

  $t4 := concat($t3,  
         if($Data/op1 castable as xs:integer)  
         then 
		 (
           concat(
		   "    (: Ограничение снизу на объем ОП :)", "&#13;",  	   
		   "    and (xs:integer($Computer/op) >= xs:integer('", $Data/op1, "')) ", "&#13;"
		   )
         )
		 else ("")),
		 
  $t5 := concat($t4,  
         if($Data/op2 castable as xs:integer)  
         then 
		 (
           concat(
		   "    (: Ограничение сверху на объем ОП :)", "&#13;",  	   	   
		   "    and (xs:integer($Computer/op) <= xs:integer('", $Data/op2, "')) ", "&#13;"
		   )
         )
		 else ("")),		 

  $t6 := concat($t5,  
         if($Data/server = "nofilter") then ("") 
         else 
		 (
           concat(
		   "    (: Проверка признака сервера :)", "&#13;",  	   	   
		   "    and ($Computer/server = '", $Data/server, "') ", "&#13;"
		   )
         )),		 

  (: Проверка того, что в заданном диапазоне было хотя бы одно обновление :)
  (: Проверка что существует хотя бы один элемент upgrade, дата которого соответственно больше или меньше заданной границы :)
  (: Здесь могла быть более сложная проверка на пересечение диапазона в запросе и диапазона обновлений :)
  $t7 := concat($t6,  
         if($Data/upgrade_date1 castable as xs:date)  
         then 
		 (
           concat(
		   "    (: Проверка того, что хотя бы одно обновление было позже параметра :)", "&#13;",  	   	   
		   "    and (fn:exists($Computer/upgrades/upgrade[xs:date(upgrade_date) >= xs:date('", $Data/upgrade_date1, "')])) ", "&#13;"
		   )
         )
		 else ("")),		 
  $t8 := concat($t7,  
         if($Data/upgrade_date2 castable as xs:date)  
         then 
		 (
           concat(
		   "    (: Проверка того, что хотя бы одно обновление было ранее параметра :)", "&#13;",  	   		   
		   "    and (fn:exists($Computer/upgrades/upgrade[xs:date(upgrade_date) <= xs:date('", $Data/upgrade_date2, "')])) ", "&#13;"
		   )
         )
		 else ("")),		 
	 
  $OrderBy := concat($t8, 
		   "(: Сортировка :)", "&#13;",  	   
           "order by ", $Data/order_field, "  ", $Data/order_type, "  ", "&#13;"
           ),  
		   
  $Result := concat($OrderBy, 
    "(: Возвращаемые данные :)", "&#13;",  	   
    "return ", "&#13;",
    "  <computer> ", "&#13;", "  { ", "&#13;",
	"    $Computer/computer_name, ", "&#13;",
	"    $processor_name, ", "&#13;",
	"    $Computer/op, ", "&#13;",
	"    $Computer/hdd, ", "&#13;",
	"    $Computer/server, ", "&#13;",
	"    $Computer/upgrades ", "&#13;",
	"  }", "&#13;", "  </computer> ", "&#13;",
	"}", "&#13;", "</report_data> ") 
	
return $Result		 
};

(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
(: Преобразование XSLT для отчета :)
(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
declare function local:transform_report($param as node(), $report_format as xs:string) as node()*
{
(: Параметры преобразования :)
let 
  $XSLTFile := concat(xrx_example:BaseCurrentURL(), "report.xsl"),
  $params := 
    <parameters>
      <param name="report_format" value="{$report_format}"/>
    </parameters>
let $TransformRes := transform:transform($param, xs:anyURI($XSLTFile), $params)
return $TransformRes
};

(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
(: Формирование отчета :) 
(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
declare function local:Report($Data as node()) as node()*
{
let 
  $ReportFormat := xs:string($Data/report_format),
  $CurrentCssURI := xrx_example:PathURI_current_css(),
  $Query := local:MakeReportQuery($Data),
  $ReportData := util:eval($Query),
  $XSLTResult := local:transform_report($ReportData, $ReportFormat)
return 
<html xmlns="http://www.w3.org/1999/xhtml" xmlns:xforms="http://www.w3.org/2002/xforms" xmlns:ev="http://www.w3.org/2001/xml-events">
  <head>
    <meta http-equiv="Content-Type" content="text/html;charset=UTF-8"/>
    <style type="text/css">
      <!-- Добавление стилей CSS с помощью XInclude -->
      <xi:include xmlns:xi="http://www.w3.org/2001/XInclude" href="{$CurrentCssURI}"/>
    </style>
  </head>
  <body>
    <h1>Результат отчета</h1>
        <div>{$XSLTResult}</div>  
    <hr/>
    <h2>Текст запроса</h2>
    <big><big><pre>{$Query}</pre></big></big>
  </body>
</html>
};

(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
(: Сохранение условий для формирования отчета :) 
(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
declare function local:PreReport() as node()*
{
(: Если сессия не создана, то она создается :)
session:create(),
session:set-attribute($report_params_data, ""),
let 
  $PostData := request:get-data(),
  $Data := fn:root($PostData)//report_params 
return
  session:set-attribute($report_params_data, $Data)
};

(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
(: Модели данных формы :) 
(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
declare function local:CreateXFormsModels($ProcessorReadURI as xs:string, $PreReportURI as xs:string) as node()*
{
(
(: Основная модель данных :)
<xforms:model id="computer_model" xmlns:xforms="http://www.w3.org/2002/xforms" xmlns:ev="http://www.w3.org/2001/xml-events">

  <!-- Основные данные (о компьютере) -->
  <xforms:instance id="computer_instance" xmlns="">
    <report_params>
      <computer_name/>
      <processor_id/>
      <op1/>
      <op2/>
      <server>nofilter</server>
      <upgrade_date1/>
      <upgrade_date2/>
      <order_field>$computer_name</order_field>	  
	  <order_type>ascending</order_type>
      <report_format>table</report_format>	  
    </report_params>
  </xforms:instance>

  <!-- Граничные значения для ОП - целого типа -->
  <xforms:bind nodeset="//op1" type="xforms:int" />
  <xforms:bind nodeset="//op2" type="xforms:int" />

  <!-- Граничные значения для даты модернизации - поле типа дата -->
  <xforms:bind nodeset="//upgrade_date1" type="xforms:date" />
  <xforms:bind nodeset="//upgrade_date2" type="xforms:date" />
   
  <!-- Отправка формы -->
  <xforms:submission replace="none" action="{$PreReportURI}" method="post" id="submit_id" includenamespaceprefixes=""/>
</xforms:model>		
),(
(: Данные для выбора типов процессоров :)
<xforms:model id="processor_model" xmlns:xforms="http://www.w3.org/2002/xforms" xmlns:ev="http://www.w3.org/2001/xml-events">
  <xforms:instance id="processor_instance" xmlns="" src="{$ProcessorReadURI}"/>
</xforms:model>		
)
};

(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
(: Редактирование формы :) 
(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
declare function local:EditForm($ReportURI as xs:string) as node()
{
<div xmlns:xforms="http://www.w3.org/2002/xforms" xmlns:ev="http://www.w3.org/2001/xml-events">
  <table border="1" align="center" >

    <tr>	
      <th colspan="2" align="right">Подстрока, которую должно содержать наименование компьютера:</th>	
      <td>
        <xforms:input model="computer_model" ref="computer_name"/>
      </td>
    </tr>

    <tr>	
      <th colspan="2" align="right">Типы процессоров:</th>	
      <td>
        <xforms:select model="computer_model" ref="processor_id">
          <xforms:itemset model="processor_model" nodeset="//processor">
            <xforms:value ref="@id"/>
            <xforms:label ref="processor_name"/>
          </xforms:itemset>
        </xforms:select>
      </td>
    </tr>
	
    <tr>	
      <th rowspan="2" align="right">Ограничение для объема ОП</th>	
      <th align="right">Минимальное:</th>	
      <td>
        <xforms:input model="computer_model" ref="op1">
          <xforms:alert>Значение должно быть целым числом</xforms:alert>
        </xforms:input>
      </td>
    </tr>

    <tr>	
      <th align="right">Максимальное:</th>	
      <td>
        <xforms:input model="computer_model" ref="op2">
          <xforms:alert>Значение должно быть целым числом</xforms:alert>
        </xforms:input>
      </td>
    </tr>
	
    <tr>	
      <th colspan="2" align="right">Используется как сервер:</th>	
      <td>
        <xforms:select1 model="computer_model" ref="server">
          <xforms:choices>
            <xforms:item>
              <xforms:label>Нет фильтра</xforms:label>
              <xforms:value>nofilter</xforms:value>
            </xforms:item>
            <xforms:item>
              <xforms:label>Да</xforms:label>
              <xforms:value>true</xforms:value>
            </xforms:item>
            <xforms:item>
              <xforms:label>Нет</xforms:label>
              <xforms:value>false</xforms:value>
            </xforms:item>
          </xforms:choices>
		</xforms:select1>
      </td>
    </tr>

    <tr>	
      <th rowspan="2" align="right">Были модернизации в диапазоне</th>	
      <th align="right">От:</th>	
      <td>
        <xforms:input model="computer_model" ref="upgrade_date1">
          <xforms:alert>Значение должно быть датой</xforms:alert>
        </xforms:input>
      </td>
    </tr>

    <tr>	
      <th align="right">До:</th>	
      <td>
        <xforms:input model="computer_model" ref="upgrade_date2">
          <xforms:alert>Значение должно быть датой</xforms:alert>
        </xforms:input>
      </td>
    </tr>

    <tr>	
      <th colspan="3" align="center">Сортировка по полю: 	

        <xforms:select1 model="computer_model" ref="order_field">
          <xforms:choices>
            <xforms:item>
              <xforms:label>Название компьютера</xforms:label>
              <xforms:value>$computer_name</xforms:value>
            </xforms:item>
            <xforms:item>
              <xforms:label>Название процессора</xforms:label>
              <xforms:value>$processor_name</xforms:value>
            </xforms:item>
          </xforms:choices>
		</xforms:select1>

        <xforms:select1 model="computer_model" ref="order_type">
          <xforms:choices>
            <xforms:item>
              <xforms:label>по возрастанию</xforms:label>
              <xforms:value>ascending</xforms:value>
            </xforms:item>
            <xforms:item>
              <xforms:label>по убыванию</xforms:label>
              <xforms:value>descending</xforms:value>
            </xforms:item>
          </xforms:choices>
		</xforms:select1>
      </th>

    </tr>

    <tr>	
      <th colspan="2" align="right">Форматировать отчет:</th>	
      <td>
        <xforms:select1 model="computer_model" ref="report_format">
          <xforms:choices>
            <xforms:item>
              <xforms:label>таблицей</xforms:label>
              <xforms:value>table</xforms:value>
            </xforms:item>
            <xforms:item>
              <xforms:label>списком</xforms:label>
              <xforms:value>list</xforms:value>
            </xforms:item>
          </xforms:choices>
		</xforms:select1>
      </td>
    </tr>
		
    <tr>	
      <td colspan="3" align="center">

        <xforms:trigger>
          <xforms:label>Сформировать отчет</xforms:label>
          <xforms:action ev:event="DOMActivate" if="is-valid(instance('computer_instance'))">
			<!-- Данные передаются на сервер и сохраняются в сессионной переменной -->
			<xforms:send submission="submit_id" />
			<!-- Данные читаются из сессионной переменной, проверяются и передаются для формирования отчета -->		
            <xforms:load show="replace" resource="{$ReportURI}"/>
          </xforms:action>
        </xforms:trigger>
		
      </td>
    </tr>
  </table>
</div>
};

(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
(: Форма параметров отчета :) 
(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
declare function local:ReportForm() as node()
{
let
$ProcessorReadURI := xrx_example:PathURI_data_list($module_processor_id, $module_processor_ver),
(: К названию текущего скрипта добавляется параметр ?prereport=1 :)
$CurrentScriptName := xrx_example:FileNameFromCurrentURL(),
$PreReportURI := concat($CurrentScriptName, "?prereport=1"),
$ReportURI := concat($CurrentScriptName, "?report=1"),
$CurrentCssURI := xrx_example:PathURI_current_css(),
$XFormsCssURI := xrx_example:PathURI_xforms_css(),
$form :=
<html xmlns="http://www.w3.org/1999/xhtml" xmlns:xforms="http://www.w3.org/2002/xforms" xmlns:ev="http://www.w3.org/2001/xml-events">
  <head>
    <meta http-equiv="Content-Type" content="text/html;charset=UTF-8"/>
    <style type="text/css">
      <!-- Добавление стилей CSS с помощью XInclude -->
      <xi:include xmlns:xi="http://www.w3.org/2001/XInclude" href="{$CurrentCssURI}"/>
      <xi:include xmlns:xi="http://www.w3.org/2001/XInclude" href="{$XFormsCssURI}"/>
    </style>
    <!-- Модель создается в секции head -->
    { local:CreateXFormsModels($ProcessorReadURI, $PreReportURI) }
  </head>
  <body>
    <h1>Параметры отчета</h1>
    <!-- Поля редактирования формы -->
    { local:EditForm($ReportURI) }
  </body>
</html>
(: сгенерированный XForms-документ обрабатывается с помощью XSLT-преобразования для получения HTML-документа :)
return xrx_example:transform_xsltforms($form)
};

(: ++++++++++++++++++++++++++++++++++++++++++++++ :)
(: Основной запрос :) 
(: ++++++++++++++++++++++++++++++++++++++++++++++ :)

(: Если не заполнен справочник типов процессоров, то формирование отчета невозможно :)
if (xrx_example:ModuleDataCount($module_processor_id, $module_processor_ver) = 0)
then
(: Вывод сообщения об ошибке :)
(
  xrx_example:ShowErrorButton("Невозможно формирование отчета, так как не заполнен справочник типов процессоров", "", "")
)
(: Если не заполнен справочник компьютеров, то формирование отчета невозможно :)
else if (xrx_example:ModuleDataCount($module_computer_id, $module_computer_ver) = 0)
then
(: Вывод сообщения об ошибке :)
(
  xrx_example:ShowErrorButton("Невозможно формирование отчета, так как не заполнен справочник компьютеров", "", "")
)
else
(: Выполнение основных действий :)
(
let 
  $reportParam := xs:string(request:get-parameter("report","0")),
  $prereportParam := xs:string(request:get-parameter("prereport","0")),
  $DataListURI := xrx_example:PathURI_list($module_id, $module_ver)
return
  if($reportParam = '1')
    (: Формирование отчета :)
    then 
    (
      local:CheckErrorsAndMakeReport($DataListURI)
    )
  else if($prereportParam = '1')
    (: Сохранение условий для формирования отчета :)
    then 
    (
      local:PreReport()
    )
  else 
    (: Генерация формы ввода данных :)
    (
      local:ReportForm()
    )
)
(: ++++++++++++++++++++++++++++++++++++++++++++++ :)


